// C++ Modem Class (kId)
// Copyright (C) 1995 Klaus Luft

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Created by: Klaus Luft, Klaus.Luft@stud.uni-karlsruhe.de
// Created on: Fri Jan  5 20:08:25 1996
// Last modified by: Klaus Luft
// Last modified on: Tue Feb 20 20:00:25 1996
// Filename: kmodem.cpp
// Purpose: C++ Modem Class (kId)

#include "kmodem.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include <sys/ioctl.h>

kModem::kModem(char *dev) {
  /* Description..: Constructor for kModem class. Open device read-only. 
                    Set 'ok=0'-flag if it fails.
     Parameter....: char *dev - the modem device */
  ok=1;

  kModemOpen(dev);
}  

kModem::kModem() {
  /* Description..: Constructor for kModem class. If environment variable $MODEM is set
                    open this device read-only, otherwise set 'ok=0'.
     Parameter....: - */
  ok=1;
  
  if(getenv("MODEM"))                         // if $MODEM is set 
    kModemOpen(getenv("MODEM"));
  else {                                     
    sprintf(error_str, "can't find modem - no device given.");
    ok=0;
  }
}

kModem::~kModem() {
  /* Description..: Destructor for kModem class. Close modem device.
     Parameter....: - */
  close(fd_modem);
}

kModem::kModemOpen(char *dev) {
  /* Description..: Open modem device.
     Parameter....: - */
  strcpy(device, dev);
  fd_modem=open(device, O_RDONLY);           // open modemport 
  if(fd_modem == -1) {
    sprintf(error_str, "can't open device `%s'", device);
    ok=0;
  }
}

void kModem::kModemCheck() {
  /* Description..: Try to store status of the serial port in the variable 'sig'.
                    If it fails, set 'ok=0' and store error msg. in 'error_str'.
     Parameter....: - */
  int status;
  status=ioctl(fd_modem, TIOCMGET, &sig);
  if(status == -1) {
    close(fd_modem);
    kModemOpen(device);
  }
}

int kModem::kModemIsRTS() {
  /* Description..: Returns the status of RTS line. If there was an error (ok=0) 
                    it returns -1.
     Parameter....: - */
  if(ok) {
    kModemCheck();
    if(ok)
      return(!!(sig & TIOCM_RTS));
    else
      return(-1);
  }
  else
    return(-1);
}
int kModem::kModemIsCTS() {
  /* Description..: Returns the status of CTS line. If there was an error (ok=0) 
                    it returns -1.
     Parameter....: - */
  if(ok) {
    kModemCheck();
    if(ok)
      return(!!(sig & TIOCM_CTS));
    else
      return(-1);
  }
  else
    return(-1);
}
int kModem::kModemIsDSR() {
  /* Description..: Returns the status of DSR line. If there was an error (ok=0) 
                    it returns -1.
     Parameter....: - */
  if(ok) {
    kModemCheck();
    if(ok)
      return(!!(sig & TIOCM_DSR));  
    else
      return(-1);
  }
  else
    return(-1);
}
int kModem::kModemIsCD() {
  /* Description..: Returns the status of CD line. If there was an error (ok=0) 
                    it returns -1.
     Parameter....: - */
  if(ok) {
    kModemCheck();
    if(ok) 
      return(!!(sig & TIOCM_CAR));
    else
      return(-1);
  }
  else
    return(-1);
}
int kModem::kModemIsDTR() {
  /* Description..: Returns the status of DTR line. If there was an error (ok=0) 
                    it returns -1.
     Parameter....: - */
  if(ok) {
    kModemCheck();
    if(ok)
      return(!!(sig & TIOCM_DTR));
    else
      return(-1);
  }
  else
    return(-1);
}
int kModem::kModemIsRNG() {
  /* Description..: Returns the status of RNG line. If there was an error (ok=0) 
                    it returns -1.
     Parameter....: - */
  if(ok) {
    kModemCheck();
    if(ok)
      return(!!(sig & TIOCM_RNG));
    else
      return(-1);
  }
  else
    return(-1);
}

// EOF: kmodem.cc



