// Main program 
// Copyright (C) 1995 Klaus Luft

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Created by: Klaus Luft, Klaus.Luft@stud.uni-karlsruhe.de
// Created on: Tue Feb 20 20:00:25 1996
// Last modified by: 
// Last modified on: 
// Filename: main.cpp
// Purpose: main

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <qapp.h>
#include <qtstream.h>

#include "momon.h"

#define O_PRG_NAME    "momon"              // name of program
#define DEF_DEVICE    "/dev/cua1"          // default device if none specified on command line 
#define DEF_LOGFILE   "/var/adm/momon.log"        // default logfile
#define DEF_COSTFILE  "/var/adm/momon_cost.log"   // default costfile
#define DEF_CDTIME    11                          // seconds for handshaking
#define DEF_CURRENCY  "DM"                        // default currency

QTextStream cout(stdout);

void usage(int status) {
  cout << "usage: " << O_PRG_NAME << " [ options ... ]\n";
  cout << "  where options include:\n\n";
  cout << "  -h\t\t\t\tto show this message.\n";
  cout << "  -v\t\t\t\tto show the version number.\n";
  cout << "  -z {0|1|2|3}\t\t\tto set the tariff zone.\n";
  cout << "  \t\t\t\t(0=City, 1=Region50, 2=Region200, 3=Fern)\n";
  cout << "  -l <filename>\t\t\tto set the logfile name.\n";
  cout << "  -c <filename>\t\t\tto set the costfile name.\n";
  cout << "  -n\t\t\t\tto disable logging.\n";
  cout << "  -m <device>\t\t\tto set the device.\n";
  cout << "  -s <n>\t\t\tto set the seconds which the modem needs for\n";
  cout << "  \t\t\t\thandshake. Will be added to online time.\n";
  cout << "  -t\t\t\t\tto test momon (only beta version).\n";
  cout << "  \t\t\t\tInstead of CD-flag CTS-flag is used.\n";
  cout << "\n";
  cout << "  -display <display>\t\tsets the X display (default is $DISPLAY).\n";
  cout << "  -geometry <geometry>\t\tsets the geometry (only position).\n"; 
  cout << "  -[bg|background] <color>\tsets the background color.\n";
  cout << "  -[fg|foreground] <color>\tsets the foreground color.\n"; 
  cout << "  -title <title>\t\tsets the application title.\n";   
  cout << "  -name <name>\t\t\tsets the application name.\n";   
  exit(status);
}

int main(int argc, char **argv) {
  char prg_name[256];
  int akt_zone=0;            // Tarifzone (0-city)
  int logging=1;             // Logfiles (an)
  int testing=0;             // Test (aus)
  char logfile[256];           // Name des Logfiles
  char costfile[256];          // Name des Kosten-Logfiles
  char mo_dev[256];            // Device des seriellen Port
  int cdtime;                // Zeit fuer Handshake

  QApplication app(argc, argv);

  strcpy(prg_name, argv[0]);
  // store default values
  strcpy(logfile, DEF_LOGFILE);
  strcpy(costfile, DEF_COSTFILE);
  strcpy(mo_dev, DEF_DEVICE);
  cdtime=DEF_CDTIME;

  // parse arguments
  int s,dz;
  while ((s=getopt(argc, argv, "hvz:nl:c:s:m:t"))>=0) {
    switch(s) {
    case 'h': // help
      usage(0);
      break;
    case 'v': // version
      cout << "MoMon v" << VERSION << " (c) 1996 Klaus Luft.\n";
      exit(0);
      break;
    case 'z': // zone
      dz=atoi(optarg);
      if(dz<0 || dz>3) {
	warning("%s: illegal argument -- z %s\nTry `%s -h' for help.", prg_name, optarg, O_PRG_NAME);
	app.quit(1);
      }
      akt_zone=dz;
      break;
    case 'n': // no logging
      logging=0;
      break;
    case 'l': // name of logfile
      strcpy(logfile, optarg);
      break;
    case 'c': // name of costfile
      strcpy(costfile, optarg);
      break;
    case 's':	// handshake time
      cdtime=atoi(optarg);
      break;
    case 'm':	// name of device
      strcpy(mo_dev, optarg);
      break;
    case 't':	// testing
      testing=1;
      break;
    default:
      usage(0);
      break;
    }
  }
  if(optind<argc)
    usage(1);

  MomonView *mom= new MomonView;
  mom->setParameters(prg_name, akt_zone, logging, logfile, costfile, cdtime, mo_dev, testing);
  app.setMainWidget(mom);
  mom->show();
  app.exec();
  exit(0);
}

// EOF: main.cpp
