// MomonView Widget
// Copyright (C) 1995 Klaus Luft

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

// Created by: Klaus Luft, Klaus.Luft@stud.uni-karlsruhe.de
// Created on: Tue Feb 20 20:00:25 1996
// Last modified by: 
// Last modified on: 
// Filename: momon.cpp
// Purpose: 

#include <qapp.h>
#include <qfile.h>
#include <qdatetm.h>
#include <qgrpbox.h>
#include <qlabel.h>
#include <qpushbt.h>

#include "momon.h"
#include "kmodem.h"
#include "ktarif.h"

MomonView::MomonView(QWidget *parent, const char *name) : QWidget(parent, name) {
  /* Konstruktor fuer das Widget. Hier werden die Textboxen und der Button 
     erstellt. Weiter wird vereinbart, dass jede Sekunde der timerEvent 
     aufgerufen wird. */
  QString s;

  initMetaObject();		// initialize meta object

  setMaximumSize(231,51);       // not resizeable
  setMinimumSize(231,51);

  // Datum und aktuelle Uhrzeit
  msg[0] = new QLabel(this, "curTime");
  msg[0]->setFrameStyle( QFrame::Box | QFrame::Sunken );
  msg[0]->setAlignment( AlignCenter );
  msg[0]->setGeometry(0, 0, 230, 25);
  msg[0]->setFont(QFont("times",12,QFont::Bold));
  s.sprintf("MoMon v%s by Klaus Luft", VERSION);
  msg[0]->setText(s);

  // Online - Zeit
  msg[1] = new QLabel(this, "onlineTime");
  msg[1]->setFrameStyle( QFrame::Box | QFrame::Sunken );
  msg[1]->setAlignment( AlignCenter );
  msg[1]->setGeometry(0, 25, 60, 25);
  msg[1]->setFont(QFont("times",12,QFont::Bold));
  msg[1]->setText("99:99:99");

  // Sekunden bis zur naechsten Einheit
  msg[2] = new QLabel(this, "restTime");
  msg[2]->setFrameStyle( QFrame::Box | QFrame::Sunken );
  msg[2]->setAlignment( AlignCenter );
  msg[2]->setGeometry(60, 25, 30, 25);
  msg[2]->setFont(QFont("times",12,QFont::Bold));
  msg[2]->setText("999");

  // Verbrauchtes Geld fuer aktuellen Anruf
  msg[3] = new QLabel(this, "money");
  msg[3]->setFrameStyle( QFrame::Box | QFrame::Sunken );
  msg[3]->setAlignment( AlignCenter );
  msg[3]->setGeometry(90, 25, 50, 25);
  msg[3]->setFont(QFont("times",12,QFont::Bold));
  msg[3]->setText("99.99");

  // Gesamtes Geld (aus vorherigen Anrufen)
  msg[4] = new QLabel(this, "total");
  msg[4]->setFrameStyle( QFrame::Box | QFrame::Sunken );
  msg[4]->setAlignment( AlignCenter );
  msg[4]->setGeometry(140, 25, 50, 25);
  msg[4]->setFont(QFont("times",12,QFont::Bold));
  msg[4]->setText("99.99");

  // Button zum Aendern der Tarifzone
  butt = new QPushButton(this, "zone");
  butt->setGeometry(191, 26, 39, 24);
  butt->setFont(QFont("times",12,QFont::Bold));
  butt->setText("City");
  connect(butt, SIGNAL(clicked()), this, SLOT(buttClicked()));

  currTimer=startTimer(1000);   // call MomonView::timerEvent every second
}

void MomonView::timerEvent(QTimerEvent *e) {
  /* Handles timer event for the widget. */
  showTime();
}

void MomonView::buttClicked() {
  /* Diese Funktion wird jedesmal aufgerufen, wenn der Button gedrueckt wird */
  count++;
  if(count==4) 
    count=0;
  setZoneButt(zones(count));
}

void MomonView::setZoneButt(zones z) {
  /* Diese Funktion setzt den Text in dem Button fuer die Tarifzone */
  akt_zone=z;

  switch(z) {
  case city:
    butt->setText("City");
    break;
  case region50:
    butt->setText("R50");
    break;
  case region200:
    butt->setText("R200");
    break;
  case fern:
    butt->setText("Fern");
    break;
  }
}

void MomonView::setParameters(char pname[256], int zone, int log, char lfile[256], char cfile[256], int cdt, char mdev[256], int testing) {
  /* Diese Funktion setzt die Default-Werte, bzw. die Werte, die der User als Parameter
     dem Programm uebergeben hat und oeffnet das Modem-Device. */

  // misc variable settings
  off_printed=0;
  old_cd=0;
  abs_money=0;
  fehlertol=0;
  prepToLog=0;

  // store function's arguments to private class variables
  strcpy(prg_name, pname);
  strcpy(logfile, lfile);
  strcpy(costfile, cfile);
  akt_zone=zones(zone);
  logging=log;
  cd_time=cdt;
  count=zone;
  test=testing;

  setZoneButt(akt_zone);

  modem=kModem(mdev);          // init modem
  if(!modem.kModemStatus()) {
    warning("%s: %s", prg_name, modem.kModemGetErrorStr());
    qApp->quit(1);
  }
}

void MomonView::showTime() {
  /* Update der Anzeige und Aufruf der Logging-Funktion */
  // update clock 
  cur_time_str = QDateTime::currentDateTime().toString();
  msg[0]->setText(cur_time_str);

  if(test)  // check CTS-line
    cd_flag=modem.kModemIsCTS();
  else      // check CD-line
    cd_flag=modem.kModemIsCD();

  if(cd_flag == -1) {
    warning("%s: %s", prg_name, modem.kModemGetErrorStr());
    qApp->quit(1);
  }
  
  // if carrier detect/lost write to logfile - start online time
  // Wenn das Modem ein/ausgeschaltet, bzw. das Programm gestartet wird,
  // kommt ein Impuls ueber die CD-Leitung. 'prepToLog' und 'fehlertol' 
  // sorgen dafuer, dass erst 5 sek. mit dem Logging gewartet wird. 
  // Besteht nach diesen 5 sek. der Carrier immer noch, wird in die
  // Logfiles geschrieben.
  if(cd_flag && prepToLog && logging) 
    fehlertol--;
  if(fehlertol==0 && prepToLog && logging) {
    writeLog(0);
    prepToLog=0;
  }

  if(old_cd!=cd_flag) { 
    if(logging && !prepToLog) {
      cd_log_time=cur_time_str;
      prepToLog=1;
      fehlertol=cd_flag*5;
    }
    else
      prepToLog=0;

    if(cd_flag) {
      tarif=kTarif(cd_time);   // starte Tarifabrechnung
      off_printed=0;                    
    }
    old_cd=cd_flag;
  }

  // now compute/update online/rest time 
  if(cd_flag) {
    tarif.kTarifUpdate();
    int rest = tarif.kTarifGetRTime(akt_zone);
    money = tarif.kTarifGetMoney(akt_zone);
    abs_money = ges_money+money;

    o_time=tarif.kTarifGetOnline();
    long dummy_time=o_time;
    int hour=int(dummy_time/3600);
    dummy_time=dummy_time%3600;
    int min=int(dummy_time/60);
    dummy_time=dummy_time%60;
    int sec=dummy_time;

    QString s;
    s.sprintf("%02d:%02d:%02d", hour, min, sec);
    msg[1]->setText(s);
    s.sprintf("%d", rest);
    msg[2]->setText(s);
    s.sprintf("%.02f", money);
    msg[3]->setText(s);
    s.sprintf("%.02f", abs_money);
    msg[4]->setText(s);
  }
  else {
    if(!off_printed) {
      msg[1]->setText("offline");
      msg[2]->setText("0");
      msg[3]->setText("0.00");
      msg[4]->setText("0.00");
      off_printed=1;
    }
  }
}

void MomonView::writeLog(int x) {
  /* Diese Funktion schreibt in die Logfiles */
  QString s;

  QFile out(logfile);
  
  if(!out.open(IO_WriteOnly | IO_Append)) {
    warning("%s: can't write to logfile `%s'.", prg_name, logfile);
    qApp->quit(1);
  }

  if(cd_flag) {
    s="Carrier detected on "+cd_log_time+"\n";
    out.writeBlock(s, s.length());
  }
  else {
    long dummy_time=o_time;
    int hour=int(dummy_time/3600);
    dummy_time=dummy_time%3600;
    int min=int(dummy_time/60);
    dummy_time=dummy_time%60;
    int sec=dummy_time;

    if(x) { // Ctrl-C pressed
      s="MoMon break      on "+cd_log_time+"\n";
      out.writeBlock(s, s.length());
      s.sprintf("Online: %02d:%02d:%02d\t\tCost: %.02f\n", hour, min, sec, money);
      out.writeBlock(s, s.length());
      s="--------------------------------------------\n";
      out.writeBlock(s, s.length());
    }
    else {
      s="Carrier lost     on "+cd_log_time+"\n";
      out.writeBlock(s, s.length());
      s.sprintf("Online: %02d:%02d:%02d\t\tCost: %.02f\n", hour, min, sec, money);
      out.writeBlock(s, s.length());
      s="--------------------------------------------\n";
      out.writeBlock(s, s.length());
    }
  }
  out.close();

  if(cd_flag) {
    QFile in(costfile);
    if(in.open(IO_ReadOnly)) {
      in.readLine(s, 255);
      ges_money=s.toFloat();
      in.close();
    }
    else 
      ges_money=0;
  }
  else {
    QFile out2(costfile);
    if(!out2.open(IO_WriteOnly)) {
      warning("%s: can't write to costfile `%s'.", prg_name, costfile);
      qApp->quit(1);
    }
    s.sprintf("%.02f", abs_money);
    out2.writeBlock(s, s.length());
    out2.close();
  }
}

// EOF: momon.cpp

